/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor;

import eu.agrosense.client.grid.Grid;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import nl.cloudfarming.client.model.SensorMetadata;
import org.openide.filesystems.FileObject;

/**
 *
 * @author Timon Veenstra
 */
public abstract class SensorProcessor {

    private static final String PROPERTY_MONTH = "MM";
    private static final String PROPERTY_YEAR = "YY";
    private static final String PROPERTY_YEAR_INTERNATIONAL = "YYYY";
    private static final String PROPERTY_DAY = "DD";
    
    /**
     * Method which  will try to create a date from the filename of a file
     * 
     * FIXME: whe no longer have a sensorfile object. Is this still needed.
     * 
     * @param fileName: filename to be created to date
     * @return: the date wich is created from the file name
     * @throws MalformedFileFormatException will be thrown when dateFormatStr does not contain:
     * PROPERTY_DAY
     * PROPERTY_MONTH
     * PROPERTY_YEAR
     * @throws ParseException
     */

    Date getDateFromFileName(FileObject fileObject) throws MalformedFileFormatException, ParseException {
        String dateFormatStr = getFileDateFormat(); // getFileDateFormat() must be overriden for each specific processor
        String fileName = fileObject.getName();
        assert !dateFormatStr.isEmpty();

        if (dateFormatStr.contains(PROPERTY_DAY) && dateFormatStr.contains(PROPERTY_MONTH) && dateFormatStr.contains(PROPERTY_YEAR)) {
            int numberOfYearCharacters = PROPERTY_YEAR.length();
            if (dateFormatStr.contains(PROPERTY_YEAR_INTERNATIONAL)) {
                numberOfYearCharacters = PROPERTY_YEAR_INTERNATIONAL.length();
            }

            String day = fileName.substring(dateFormatStr.indexOf(PROPERTY_DAY), dateFormatStr.indexOf(PROPERTY_DAY) + 2);
            String month = fileName.substring(dateFormatStr.indexOf(PROPERTY_MONTH), dateFormatStr.indexOf(PROPERTY_MONTH) + 2);
            String year = fileName.substring(dateFormatStr.indexOf(PROPERTY_YEAR), dateFormatStr.indexOf(PROPERTY_YEAR) + numberOfYearCharacters);
            StringBuilder dateBuilder = new StringBuilder(day);
            dateBuilder.append(month);
            dateBuilder.append(year);

            SimpleDateFormat date = new SimpleDateFormat("ddMMyy");
            return date.parse(dateBuilder.toString());
        } else {
            StringBuilder sb = new StringBuilder("The property which specifies file format is malformed while creating date from filename: ");
            sb.append(fileName);
            throw new MalformedFileFormatException(sb.toString());
        }
    }
    
    
    /**
     * process a sensor data file
     */
    public abstract Grid process();

    /*
     * Each sensorProcessor should have its own buildMetadataFromPreferences() method
     */
    protected abstract SensorMetadata buildMetadataFromPreferences();
    
    /**
     * Get file dateformat. 
     * @return String reflecting the format of the date inside the filename of the sensorfile.
     */
    protected abstract String getFileDateFormat();
 
    /**
     * Concrete subclasses must override this method to return the displayname of that sensorprocessor to be displayed in the log.
     * For example: "ASC-340 Cropcircle"
     * 
     * Primary use is for logging, for example: [Info] Start processing of ASC-340 Cropcircle sensor file 010710AD.CSV
     * 
     * @return 
     */
    protected abstract String getLogDisplayName();    
}
